package config

import (
	"github.com/confluentinc/cli/v3/pkg/types"
)

var SQLKeywords = types.NewSet(
	"ABS",
	"ACOS",
	"ADD",
	"ADMIN",
	"AFTER",
	"ALL",
	"ALLOW",
	"ALTER",
	"ANALYZE",
	"AND",
	"ANY",
	"ARE",
	"ARRAY",
	"ARRAY_CONTAINS",
	"AS",
	"ASC",
	"ASCII",
	"ASIN",
	"ASYMMETRIC",
	"AT",
	"ATAN",
	"ATAN2",
	"AVG",
	"BEFORE",
	"BEGIN",
	"BETWEEN",
	"BIGINT",
	"BIN",
	"BINARY",
	"BIT",
	"BLOB",
	"BOOLEAN",
	"BOTH",
	"BY",
	"BYTES",
	"CALL",
	"CALLED",
	"CALL_REMOTE_SCALAR",
	"CALL_REMOTE_SCALAR_STR",
	"CARDINALITY",
	"CASCADE",
	"CASCADED",
	"CASE",
	"CAST",
	"CATALOG",
	"CATALOGS",
	"CEIL",
	"CENTURY",
	"CHAIN",
	"CHANGELOG_MODE",
	"CHAR",
	"CHARACTER",
	"CHARACTERS",
	"CHARACTER_LENGTH",
	"CHAR_LENGTH",
	"CHECK",
	"CHR",
	"CLOB",
	"CLOSE",
	"COALESCE",
	"COLLATE",
	"COLLECT",
	"COLUMN",
	"COLUMNS",
	"COMMENT",
	"COMMIT",
	"COMPACT",
	"CONCAT",
	"CONCAT_WS",
	"CONNECT",
	"CONSTRAINT",
	"CONSTRAINTS",
	"CONSTRUCTOR",
	"CONTAINS",
	"CONVERT",
	"CONVERT_TZ",
	"COS",
	"COSH",
	"COT",
	"COUNT",
	"CREATE",
	"CROSS",
	"CUBE",
	"CUME_DIST",
	"CUMULATE",
	"CURRENT",
	"CURRENT_DATABASE",
	"CURRENT_DATE",
	"CURRENT_ROW_TIMESTAMP",
	"CURRENT_TIME",
	"CURRENT_TIMESTAMP",
	"CURRENT_WATERMARK",
	"CURSOR",
	"CYCLE",
	"DATA",
	"DATABASE",
	"DATABASES",
	"DATE",
	"DATETIME",
	"DATE_FORMAT",
	"DAY",
	"DAYOFMONTH",
	"DAYOFWEEK",
	"DAYOFYEAR",
	"DAYS",
	"DEC",
	"DECADE",
	"DECIMAL",
	"DECLARE",
	"DECODE",
	"DEFAULT",
	"DEFINE",
	"DEFINED",
	"DEGREES",
	"DELETE",
	"DENSE_RANK",
	"DESC",
	"DESCRIBE",
	"DESCRIPTOR",
	"DISTINCT",
	"DIV",
	"DOUBLE",
	"DROP",
	"E",
	"EACH",
	"ELEMENT",
	"ELSE",
	"ENCODE",
	"ENCODING",
	"END",
	"ENFORCED",
	"ENGINE",
	"EPOCH",
	"EQUALS",
	"ERROR",
	"ESCAPE",
	"ESTIMATED_COST",
	"EXCEPT",
	"EXCEPTION",
	"EXCLUDE",
	"EXCLUDING",
	"EXECUTE",
	"EXISTS",
	"EXP",
	"EXPLAIN",
	"EXTENDED",
	"EXTERNAL",
	"EXTRACT",
	"FALSE",
	"FILE",
	"FINAL",
	"FIRST",
	"FIRST_VALUE",
	"FLOAT",
	"FLOOR",
	"FOLLOWING",
	"FOR",
	"FORMAT",
	"FORTRAN",
	"FOUND",
	"FRAC_SECOND",
	"FROM",
	"FROM_BASE64",
	"FROM_UNIXTIME",
	"FULL",
	"FUNCTION",
	"FUNCTIONS",
	"GENERAL",
	"GENERATED",
	"GLOBAL",
	"GO",
	"GOTO",
	"GRANT",
	"GRANTED",
	"GREATEST",
	"GROUP",
	"GROUPING",
	"GROUPING_ID",
	"GROUPS",
	"GROUP_ID",
	"HAVING",
	"HEX",
	"HOP",
	"HOUR",
	"HOURS",
	"IF",
	"IFNULL",
	"IGNORE",
	"IMPORT",
	"IN",
	"INCLUDING",
	"INCREMENT",
	"INITCAP",
	"INNER",
	"INOUT",
	"INPUT",
	"INSERT",
	"INSTR",
	"INT",
	"INTEGER",
	"INTERSECT",
	"INTERVAL",
	"INTO",
	"INVOKER",
	"INVOKE_OPENAI",
	"IS",
	"IS_ALPHA",
	"IS_DECIMAL",
	"IS_DIGIT",
	"JAR",
	"JARS",
	"JAVA",
	"JOB",
	"JOBS",
	"JOIN",
	"JSON",
	"JSON_ARRAY",
	"JSON_EXECUTION_PLAN",
	"JSON_EXISTS",
	"JSON_OBJECT",
	"JSON_QUERY",
	"JSON_STRING",
	"JSON_VALUE",
	"KEY",
	"KEY_MEMBER",
	"KEY_TYPE",
	"LABEL",
	"LAG",
	"LANGUAGE",
	"LAST",
	"LAST_VALUE",
	"LATERAL",
	"LEAD",
	"LEADING",
	"LEAST",
	"LEFT",
	"LENGTH",
	"LEVEL",
	"LIKE",
	"LIMIT",
	"LISTAGG",
	"LN",
	"LOAD",
	"LOCAL",
	"LOCALTIME",
	"LOCALTIMESTAMP",
	"LOCATE",
	"LOG",
	"LOG10",
	"LOG2",
	"LOWER",
	"LPAD",
	"LTRIM",
	"MAP",
	"MATCH",
	"MATCH_RECOGNIZE",
	"MATCH_ROWTIME",
	"MAX",
	"MD5",
	"MEASURES",
	"MERGE",
	"METADATA",
	"MICROSECOND",
	"MILLENNIUM",
	"MILLISECOND",
	"MIN",
	"MINUS",
	"MINUTE",
	"MINUTES",
	"MINVALUE",
	"MOD",
	"MODIFIES",
	"MODIFY",
	"MODULE",
	"MODULES",
	"MONTH",
	"MONTHS",
	"MULTISET",
	"NANOSECOND",
	"NATURAL",
	"NEXT",
	"NO",
	"NONE",
	"NOT",
	"NOW",
	"NTILE",
	"NULL",
	"NULLIF",
	"NULLS",
	"NUMBER",
	"NUMERIC",
	"OF",
	"OFFSET",
	"ON",
	"ONE",
	"OPTION",
	"OPTIONS",
	"OR",
	"ORDER",
	"ORDERING",
	"OTLP_EXAMPLE_METRICS",
	"OTLP_METRICS",
	"OUT",
	"OUTER",
	"OUTPUT",
	"OVER",
	"OVERLAPS",
	"OVERLAY",
	"OVERWRITE",
	"OVERWRITING",
	"PARSE_URL",
	"PARTITION",
	"PARTITIONED",
	"PARTITIONS",
	"PASSING",
	"PAST",
	"PATH",
	"PATTERN",
	"PER",
	"PERCENT",
	"PERCENT_RANK",
	"PERIOD",
	"PI",
	"PLACING",
	"PLAN",
	"POSITION",
	"POWER",
	"PRECEDING",
	"PRESERVE",
	"PRIMARY",
	"PRIOR",
	"PRIVILEGES",
	"PROCTIME",
	"PUBLIC",
	"PYTHON",
	"PYTHON_ARCHIVES",
	"PYTHON_DEPENDENCIES",
	"PYTHON_FILES",
	"PYTHON_JAR",
	"PYTHON_PARAMETER",
	"PYTHON_REQUIREMENTS",
	"QUARTER",
	"RADIANS",
	"RAND",
	"RAND_INTEGER",
	"RANGE",
	"RANK",
	"RAW",
	"READ",
	"REGEXP",
	"REGEXP_EXTRACT",
	"REGEXP_REPLACE",
	"RELATIVE",
	"REMOVE",
	"RENAME",
	"REPEAT",
	"REPLACE",
	"RESET",
	"RESPECT",
	"RESTART",
	"RESTRICT",
	"REVERSE",
	"REVOKE",
	"RIGHT",
	"RLIKE",
	"ROLE",
	"ROLLBACK",
	"ROLLUP",
	"ROUND",
	"ROW",
	"ROWS",
	"ROW_COUNT",
	"ROW_NUMBER",
	"RPAD",
	"RTRIM",
	"SCALA",
	"SCALAR",
	"SCALE",
	"SCHEMA",
	"SECOND",
	"SECONDS",
	"SECRET",
	"SECTION",
	"SECURITY",
	"SELECT",
	"SELF",
	"SERVER",
	"SERVER_NAME",
	"SESSION",
	"SET",
	"SETS",
	"SHA1",
	"SHA2",
	"SHA224",
	"SHA256",
	"SHA384",
	"SHA512",
	"SHOW",
	"SIGN",
	"SIMILAR",
	"SIMPLE",
	"SIN",
	"SINH",
	"SIZE",
	"SKIP",
	"SLIDE",
	"SMALLINT",
	"SOURCE",
	"SOURCE_WATERMARK",
	"SPACE",
	"SPLIT_INDEX",
	"SQRT",
	"START",
	"STATE",
	"STATEMENT",
	"STATIC",
	"STDDEV",
	"STDDEV_POP",
	"STDDEV_SAMP",
	"STEP",
	"STRING",
	"STRUCTURE",
	"STR_TO_MAP",
	"STYLE",
	"SUBSTR",
	"SUBSTRING",
	"SUM",
	"SYMMETRIC",
	"SYSTEM",
	"SYSTEM_TIME",
	"SYSTEM_USER",
	"TABLE",
	"TABLES",
	"TABLESAMPLE",
	"TAN",
	"TANH",
	"TEMPORARY",
	"THEN",
	"TIME",
	"TIMECOL",
	"TIMESTAMP",
	"TIMESTAMPADD",
	"TIMESTAMPDIFF",
	"TIMESTAMP_LTZ",
	"TINYINT",
	"TO",
	"TO_BASE64",
	"TO_DATE",
	"TO_TIMESTAMP",
	"TO_TIMESTAMP_LTZ",
	"TRAILING",
	"TRANSFORM",
	"TRIM",
	"TRUE",
	"TRUNCATE",
	"TRY_CAST",
	"TUMBLE",
	"TYPE",
	"TYPEOF",
	"UNBOUNDED",
	"UNDER",
	"UNION",
	"UNIQUE",
	"UNIX_TIMESTAMP",
	"UNKNOWN",
	"UNLOAD",
	"UNNEST",
	"UPPER",
	"UPSERT",
	"USAGE",
	"USE",
	"USER",
	"USING",
	"UTF16",
	"UTF32",
	"UTF8",
	"UUID",
	"VALUE",
	"VALUES",
	"VARBINARY",
	"VARCHAR",
	"VARIANCE",
	"VAR_POP",
	"VAR_SAMP",
	"VERSION",
	"VIEW",
	"VIEWS",
	"VIRTUAL",
	"WATERMARK",
	"WATERMARKS",
	"WEEK",
	"WEEKS",
	"WHEN",
	"WHERE",
	"WINDOW",
	"WITH",
	"WITHIN",
	"WITHOUT",
	"WORK",
	"WRAPPER",
	"YEAR",
	"YEARS",
	"ZONE",
)
